#!/bin/bash

. /usr/local/Reductor/etc/const
. /etc/init.d/functions
. $CONFIG
. $MAINDIR/etc/messages
. $MENULIB

ERRORFILE="/opt/reductor_web/app/var/errors"
RKN_LIST_MAX_DELTA_SECS=$((12*60*60))
RKN_ENDPOINT="vigruzki.rkn.gov.ru"
MAX_PROCESS_REDUCTOR=100
MAX_PROCESS_TOTAL=300

fix_failed_test() {
	echo
	echo "Попытка исправить $cmd:"
	case $cmd in
		check_module_writable )
			check_module_writable_fix
			;;
		check_network_manager )
			check_network_manager_error
			;;
		check_list_data )
			check_list_data_error
			;;
		check_web_version )
			check_web_version_fix
			;;
		check_dns )
			check_dns_fix
			;;
		check_new_reg )
			check_new_reg_error
			;;
		check_new_reg_message )
			check_new_reg_message_error
			;;
		check_process_count )
			check_process_count_error
			;;
		check_dropped_packets )
			check_dropped_packets_error
			;;
		check_suds_installed )
			check_suds_installed_fix
			;;
		check_bridge )
			check_bridge_error
			;;
		is_module_load )
			is_module_load_fix
			;;
		check_proc_values )
			check_proc_values_error
			;;
		check_tcpdump_http_mirror )
			check_tcpdump_http_mirror_error
			;;
		check_cert )
			check_cert_error
			;;
		check_key_in_cert )
			check_key_in_cert_error
			;;
		check_total_packets )
			check_total_packets_error
			;;
		check_https_cache )
			check_https_cache_fix
			;;
		check_signatures_cache )
			check_signatures_cache_fix
			;;
		check_block_fact )
			check_block_fact_error
			;;
		check_url_modify )
			check_url_modify_error
			;;
		check_forbidden_default_url )
			fix_forbidden_default_url
			;;
		check_list_actuality )
			check_list_actuality_fix
			;;
		check_activation )
			check_activation_fix
			;;
		check_disk_space )
			check_disk_space_error
			;;
		check_cert_date )
			check_cert_date_error
			;;
		check_network_errors )
			check_network_errors_error
			;;
		check_crond_running )
			check_crond_running_fix
			;;
		* )
			echo "Для этого автоматическое исправление ещё не придумано."
			;;
	esac
	echo
	return 0
}

run_test() {
	local quiet=0
	if [ "$1" = 'quiet' ]; then
		local quiet=1
		shift
	fi
	cmd="$1"
	shift
	descr="$*"
	if [ "$markdown" = '1' ]; then
		echo -n "$descr"
	else
		echo -n "${descr##*# }"
	fi
	$cmd
	retval=$?

	[ "$quiet" = '1' ] && return $retval

	if [ "$retval" = '0' ]; then
		echo_success
		echo
		return $retval
	fi

	echo ${descr####} >> $ERRORFILE
	echo_failure
	[ "$markdown" = '1' ] && echo
	[ "$markdown" = '1' ] && echo '<pre>'
	fix_failed_test "$1"
	[ "$markdown" = '1' ] && echo '</pre>'
	echo
	return $retval
}

check_autostart() {
	! LANG="ru_RU.UTF-8" chkconfig --list | egrep "reductor|reductor_web|nginx|network|crond" | grep -v 'debug' | grep -v "3:вкл.*4:вкл.*5:вкл"
}

check_autostart_error() {
	echo 'Если вы не отключали сервисы reductor|reductor_web|nginx|crond осознано, выполните команду'
	echo 'for service in reductor reductor_web nginx crond; do chkconfig --level 345 $service on; done'
	echo 'Если сервис отключен намеренно и на постоянно, можете отключить эту проверку хуком диагностики, в котором достаточно написать:'
	echo 'check_autostart() { :; }'
}

check_network_manager() {
	! pgrep NetworkManager
}

check_network_manager_error() {
	echo "Вместо NetworkManager рекомендуется использовать /etc/init.d/network."
	echo "Сеть нужно вручную или с помощью мастера настройки сети настроить в /etc/sysconfig/network-scripts"
}

check_module_writable() {
	echo ping > $PROCLIST && ! dmesg | grep -v "forbidden:" | tail -10 | grep 'wrong data put in reductor'
}

check_module_writable_fix() {
	echo "Повторно обрабатываем списки, перезагружаем модуль, есть вероятность повреждения памяти кривыми данными."
	echo "Скорее всего ошибка одноразовая и больше не повторится."
	$BINDIR/prepare_lists.sh
	dmesg -c | tail -10
	$BINDIR/stop.sh
	sleep 1
	$BINDIR/start.sh
}

check_forbidden_default_url() {
	[ -z "${filter['forbidden_url']}" ] && return 0
	[[ "${filter['forbidden_url']}" == *helpdesk.carbonsoft.ru*  ]] && return 1
	return 0
}

fix_forbidden_default_url() {
	local NEW_DEFAULT_URL='http://deny.carbonsoft.ru'
	echo "  - Меняем адрес заглушки с ${filter['forbidden_url']} на $NEW_DEFAULT_URL"
	filter['forbidden_url']="$NEW_DEFAULT_URL"
	save_and_exit
}

check_list_data() {
	if [ -f $LISTDIR/our.list ]; then
		if grep -v ^http:// $LISTDIR/our.list > $TMPDIR/list.tmp; then
			printf "\n# our.list\n"
			cat $TMPDIR/list.tmp
			return 1
		fi
	fi
	if [ -f $LISTDIR/our.iplist ]; then
		if egrep -v ^$ip_regex$ $LISTDIR/our.iplist > $TMPDIR/list.tmp; then
			printf "\n# our.iplist\n"
			cat $TMPDIR/list.tmp
			return 2
		fi
	fi
	return 0
}

check_list_data_error() {
	echo "Один из собственных списков содержит некорректные данные."
}

check_web_version() {
	local version
	local latest_hotfix
	version="$(rpm -q reductor_web | sed -e 's/el6.*//; s/[^0-9]//g')"
	latest_hotfix="113895"
	if [ -z "$version" ]; then
		return 0
	fi
	if [ "$version" -lt "$latest_hotfix" ]; then
		return 1
	fi
	return 0
}

check_web_version_fix() {
	/usr/local/Reductor/bin/install_web_interface.sh
}

check_url_count() {
	if [ ! -f $LISTDIR/rkn.list ]; then
		return 0
	fi
	count="$(wc -l < $LISTDIR/rkn.list)"
	count_real="$(grep ^URL $PROCLIST | egrep -o [0-9]+)"
	[ "$count_real" -gt "$((count))" ]
}

check_url_count_fix() {
	if ! $BINDIR/load.sh; then
		echo "Пробуем обработать списки и рестартануть редуктор"
		$BINDIR/prepare_lists.sh
		$BINDIR/restart.sh
		return
	fi
	echo "Помогла простая перезаливка URLов"
	return 0
}

check_new_reg_message() {
	local messages
	messages="$($BINDIR/reg_util message)"
	if [ -n "$messages" ]; then
		$BINDIR/reg_util activate
	fi
	messages="$($BINDIR/reg_util message)"
	if [ -n "$messages" ]; then
		return 2
	fi
}

check_new_reg_message_error() {
	local messages
	messages="$($BINDIR/reg_util message)"
	echo
	echo "$messages"
	echo "$messages" >> $ERRORFILE
}

check_new_reg() {
	$BINDIR/nice_json /var/reg/CRB-Reductor.reg &>/dev/null || return 1
	$BINDIR/nice_json /var/reg/CRB-Reductor.key &>/dev/null || return 2
	return 0
}

check_https_cache() {
	[ "${filter['https_by_ip']}" = '0' ] && return 0
	[ "${filter['https_by_nslookup']}" = '0' ] && return 0
	if ! $BINDIR/nice_json $CACHEDIR/https_resolv.json > /dev/null; then
		sleep 5
		$BINDIR/nice_json $CACHEDIR/https_resolv.json > /dev/null
	fi
	return
}

check_https_cache_fix() {
	sleep 2
	log 'checking https_resolv.json again'
	if ! check_https_cache; then
		log 'checking https_resolv.json failed'
		mv -v $CACHEDIR/https_resolv.json "$CACHEDIR/https_resolv.json.bad.$(date +%s)"
	else
		log 'checking https_resolv.json ok after 2 seconds wait'
	fi
	$BINDIR/reload_ip.sh
}

check_signatures_cache() {
	grep -q "http://.*ololo.fm.*http://" $CACHEDIR/signatures.cache && return 1
	grep -q "http://vk.com.*http://" $CACHEDIR/signatures.cache && return 2
	grep -qi "newcamd525://" $CACHEDIR/signatures.cache && return 3
	if grep -v ^$ $CACHEDIR/signatures.cache | grep -v http://; then
		echo "Не http-записи в signatures-cache" >> $ERRORFILE
		return 4
	fi
	return 0
}

check_signatures_cache_fix() {
	echo > $CACHEDIR/signatures.cache
	$BINDIR/update_signatures.sh
}

check_suds_installed() {
	. $CHROOTDIR/virtualenv/bin/activate
	python -c 'import suds'
	retval=$?
	deactivate
	return $retval
}

check_suds_installed_fix() {
	yum -y erase suds || true
	pip install suds
}

check_dns() {
	resolv_conf_restore_normal_settings
	nslookup $RKN_ENDPOINT >/dev/null
}


check_dns_fix() {
	local fallback_dns="${misc['diagnostic.fallback_dns']:-8.8.8.8}"

	echo "Обнаружена проблема с резолвом $RKN_ENDPOINT"
	sleep 5

	if check_dns; then
		echo "Проблема с резолвом разовая, но нужно разобраться, если повторится"
		return 1
	fi

	if [ ! -s /etc/resolv.conf ]; then
		echo "Обнаружен пустой resolv.conf, добавим туда $fallback_dns"
		echo "nameserver $fallback_dns" > /etc/resolv.conf
	fi

	if ! nslookup $RKN_ENDPOINT $fallback_dns; then
		echo "Не удаётся разрезолвить выгрузки резервным DNS $fallback_dns"
		return 2
	fi

	if grep -q $fallback_dns /etc/resolv.conf; then
		echo "Резервный DNS $fallback_dns уже прописан и это не помогает"
		return 3
	fi

	if [ -s /etc/resolv.conf.bk_reductor ]; then
		echo "Резервный DNS уже включен, проблема сохраняется"
		return 4
	fi

	resolv_conf_add_fallback_dns

	echo "Проверяем исправление:"

	if ! nslookup $RKN_ENDPOINT; then
		echo "Не удалось исправить резолв выгрузок"
		return 5
	fi

	return 0
}

resolv_conf_restore_normal_settings() {
	local conf='/etc/resolv.conf'
	[ -s $conf.bk_reductor ] && mv -f $conf.bk_reductor $conf
}

resolv_conf_add_fallback_dns() {
	local fallback_dns="${misc['diagnostic.fallback_dns']:-8.8.8.8}"
	echo "# Прописываю резервный DNS $fallback_dns в /etc/resolv.conf"
	echo "nameserver $fallback_dns" > $TMPDIR/resolv.conf
	cat /etc/resolv.conf >> $TMPDIR/resolv.conf
	cp -v /etc/resolv.conf /etc/resolv.conf.bk
	cat $TMPDIR/resolv.conf > /etc/resolv.conf
}

check_network_errors() {
	local TMPFILE=$TMPDIR/check_network_errors
	ip -s -s l sh label eth* | egrep '([0-9]+[^a-z:]+){6}' | egrep "([0-9]+){6}" > $TMPFILE
	while read packets _ errors _; do
		[ $((errors * 100 / (packets+1))) -gt 1 ] && return 1
	done < $TMPFILE
	rm -f $TMPFILE
	return 0
}

find_all_br() {
	brctl show | egrep -o "^br[0-9]+"
}

check_dropped_packets() {
	! tr ':' ' ' < /proc/net/dev | grep "[0-9]" | awk '{print $5}' > $TMPDIR/check_dropped_packets_new
	if [ ! -f $TMPDIR/check_dropped_packets_prev ]; then
		mv $TMPDIR/check_dropped_packets_new $TMPDIR/check_dropped_packets_prev
		return 0
	fi
	cmp $TMPDIR/check_dropped_packets_new $TMPDIR/check_dropped_packets_prev > /dev/null && return 0
	mv $TMPDIR/check_dropped_packets_new $TMPDIR/check_dropped_packets_prev
	grep -v '^0$' $TMPDIR/check_dropped_packets_prev
}

check_bridge() {
	[ "${misc['skipnetworkcheck']}" = '1' ] && return 0
	ip link | grep -q "br[0-9]"
}

get_bridge_slave() {
	while read bridge; do
		brctl show $bridge | awk '{print $4}' | tail -n +2
	done | grep -v "^$"
}

have_ip() {
	echo
	while read device; do
		echo -n "- Проверка наличия IP на $device:"
		if ! ip a s $device | grep -qw inet; then
			echo_failure
			echo
			return 1
		else
			echo_success
			echo
		fi
	done
}

check_all_slave_have_ip() {
	[ "${misc['skipnetworkcheck']}" = '1' ] && echo && return 0
	find_all_br | get_bridge_slave | have_ip
}

is_module_load() {
	[ -f /proc/net/ipt_reductor/block_list ]
}

check_tcpdump_http_mirror() {
	[ "${misc['nomirror']:-0}" = '1' ] && return 0
	timeout -s 15 10s tcpdump -c 1 -nni any tcp dst port 80 &>/dev/null
}

check_proc_values() {
	grep -q '1' /proc/sys/net/ipv4/ip_forward || return 1
	grep -q '1' /proc/sys/net/bridge/bridge-nf-call-iptables || return 1
	grep -q '2' /proc/sys/net/ipv4/conf/{default,eth*}/rp_filter || return 1
}

check_cert() {
	[ "${autoupdate['skip_sign_request']:-0}" = '1' ] && return 0
	[ "${autoupdate['own']:-0}" = '1' ] && return 0
	[ "${autoupdate['enabled']}" != '1' ] && return 0
	[ -s $USERDIR/provider.pem ]
}

check_key_in_cert() {
	[ "${autoupdate['skip_sign_request']:-0}" = '1' ] && return 0
	[ "${autoupdate['own']:-0}" = '1' ] && return 0
	[ "${autoupdate['enabled']}" != '1' ] && return 0
	grep -q 'PRIVATE KEY' $USERDIR/provider.pem
}

check_list_actuality() {
	[ "${autoupdate['enabled']}" != '1' -a "${autoupdate['own']}" = '0' ] && return 0
	local file=$SSLDIR/php/register.zip
	if ! [ -f $file ]; then
		log "Отсутствует $file" >> $LOGFILE
		local file="$DUMPXML"
		if ! [ -f $file ]; then
			log "Отсутствует $file" >> $LOGFILE
			return 1
		fi
	fi
	last_modify_date=$(stat -c "%Y" $file)
	current_date=$(date +%s)
	delta="$((current_date - last_modify_date))"
	[ -z "$last_modify_date" ] && return 1
	[ -z "$current_date" ] && return 2
	if ! [ "$delta" -lt "$RKN_LIST_MAX_DELTA_SECS" ]; then
		log dates: last_modify_date=$last_modify_date current_date=$current_date delta=$delta
		return 3
	fi
}

check_block_fact() {
	date +%H | grep -q ^0 && return 0
	if [ "$(grep "Total packets checked" $PROCLIST | tr -d '[A-Za-z: ]')" -gt '15000000' ]; then
		! grep -q 'Match.*\<0\>$' $PROCLIST
	fi
}

check_total_packets() {
	[ "${misc['nomirror']:-0}" = '1' ] && return 0
		! grep -q 'Total packets checked:\s*0' $PROCLIST
}

check_url_modify() {
	[ -z "${autoupdate['url']}" ] && return 1
	[[ "${autoupdate['url']}" != *docs.carbonsoft.ru* ]] && return 0
	if grep -q carbonsoft /etc/hosts; then
		echo
		echo "WARNING: carbonsoft в /etc/hosts"
		echo "Может привести к проблемам с доступом к серверу активации"
		echo "# /etc/hosts"
		echo
		cat /etc/hosts
		echo
		return 2
	fi
	[ "${autoupdate['url']}" = 'http://docs.carbonsoft.ru:1488/cache/' ]
}

check_activation() {
	[ "$(get_reductor_state)" = 'Активирован' ]
}

check_disk_space() {
	percent="$(df | grep '/$' | grep -o "[0-9]*%" | tr -d '%')"
	[ "$percent" -le '85' ]
}

check_cert_date() {
	[ "${autoupdate['skip_sign_request']:-0}" = '1' -o "${autoupdate['own']:-0}" = '1' ] && return 0
	days_before_finish=$((($(date +%s --date="$(/usr/local/Reductor/bin/check_cert_date auto | grep notAfter | cut -d '=' -f2)") - $(date +%s))/86400))
	echo -n " $days_before_finish дней"
	[ "$days_before_finish" -gt '14' ]
}

check_cert_date_error() {
	echo "Нужно приобрести новый сертификат"
}

check_crond_running() {
	pgrep crond &>/dev/null
}

check_crond_running_fix() {
	/etc/init.d/crond restart
}

check_crond_count() {
	[ "$(</var/run/crond.pid)" = "$(ps aux | grep -w crond$ | awk '{print $2}')" ]
}

check_process_count() {
	if [ "$(ps aux | egrep -c "reload_ip.sh|ip_update.sh|ip_update.py")" -gt ${MAX_PROCESS_REDUCTOR:-100} ]; then
		echo "- Убиваем зависшие процессы ip_update"
		ps aux | egrep "reload_ip.sh|ip_update.sh|ip_update.py" | awk '{print $2}' | xargs kill
	fi
	[ "$(ps ax | fgrep -vc '[')" -lt ${MAX_PROCESS_TOTAL:-300} ]
}

dummy() {
	return 1
}

use_hook() {
	hook=$HOOKDIR/${0##*/}
	[ -x $hook ] && . $hook
}

client_diagnostic_hook() {
	return 0
}

main() {
	local retval=0
	[ "$1" = 'skip' ] && return 0
	[ "$1" != 'auto' ] && clear || set_consoletype_serial
	mkdir -p "${ERRORFILE%/*}"
	> "$ERRORFILE"

	if [ "$markdown" = '0' ]; then
		echo "Проверка распространённых ошибок настройки, мешающих работе Carbon Reductor"
		echo
	fi
	run_test check_activation "## Проверка состояния активации" || ((retval++))
	run_test check_bridge "## Проверка настроенного bridge-интерфейса" || ((retval++))
	if check_bridge quiet; then
		run_test check_all_slave_have_ip "## Проверка наличия IP адресов на интерфейсах в bridge" || ((retval++))
	fi
	run_test is_module_load "## Подгрузка модуля" || ((retval++))
	run_test check_proc_values "## Проверка proc-параметров" || ((retval++))
	run_test check_crond_running "## Проверка запущенного crond" || ((retval++))

	if [ "${autoupdate['enabled']}" = '1' ]; then
		run_test check_cert "## Наличие сертификата" || ((retval++))
		if check_cert; then
			run_test check_key_in_cert "## Наличие закрытого ключа в provider.pem" || ((retval++))
		fi
		if check_cert; then
			run_test check_cert_date "## Время до конца работы ключа provider.pem" || ((retval))
		fi
	fi

	run_test check_dns "## Проверка работоспособности DNS-сервера" || ((retval++))
	run_test check_list_actuality "## Проверка актуальности списков" || ((retval++))
	run_test check_total_packets "## Наличие проверенных пакетов" || ((retval++))
	if check_total_packets; then
		run_test check_block_fact "## Наличие факта блокировки (если были попытки)" || ((retval++))
	fi
	run_test check_tcpdump_http_mirror "## Наличие трафика в tcpdump" || ((retval++))
	run_test check_url_modify "## Исправление URL сервера сигнатур" || ((retval++))
	run_test check_suds_installed "## Проверяем что suds установлен" || ((retval++))
	run_test check_disk_space "## Проверка свободного места на диске" || ((retval++))
	run_test check_network_errors "## Проверка ошибок обработки пакетов на сетевых картах" || ((retval++))
	run_test check_dropped_packets "## Проверка дропов пакетов на сетевых картах" || ((retval++))
	run_test check_https_cache "## Проверка состояния кэша https" || ((retval++))
	run_test check_signatures_cache "## Проверка состояния кэша сигнатур" || ((retval++))
	run_test check_new_reg "## Проверка новой системы регистрации/активации" || ((retval++))
	if check_new_reg; then
		run_test check_new_reg_message "## Проверка активации" || ((retval++))
	fi
	run_test check_process_count "## Проверка числа запущенных процессов" || ((retval++))
	run_test check_url_count "## Проверка числа загруженных URL" || ((retval++))
	run_test check_crond_count "## Проверка числа запущенных crond" || ((retval++))
	run_test check_web_version "## Проверка устаревшей версии веб-интерфейса" || ((retval++))
	run_test check_list_data "## Проверка содержимого собственных списков" || ((retval++))
	run_test check_forbidden_default_url "## Проверяем URL страницы-заглушки" || ((retval++))
	run_test check_module_writable "## Проверяем работоспособность модуля" || ((retval))
	run_test check_network_manager "## Проверяем отсутствие NetworkManager" || ((retval))
	run_test check_autostart "## Проверяем автоматический старт сервисов" || ((retval))
	client_diagnostic_hook
	if [ "$1" != 'auto' ]; then
		echo
		read -p "Нажмите ENTER"
	fi
	if [ "$retval" -gt '0' ]; then
		echo 'Внимание!'
		echo 'Возможна некорректная работа продукта!'
	fi
	if [ -s "$ERRORFILE" ]; then
		{
			echo "Ошибки:"
			cat "$ERRORFILE"
		} > $TMPDIR/errorfile
		mv $TMPDIR/errorfile $ERRORFILE
	fi
	return $retval
}

use_hook

markdown=0
[ "$1" = 'markdown' ] && markdown=1 && shift
main "$@" 2>&1
