#!/usr/bin/python
# -*- coding: utf-8 -*-
"""
Утилита для разбора XML с единым реестром запрещённых сайтов
Реестр преобразуется в простой текстовый файл, подходящий для Carbon Reductor
"""

__author__ = 'oleg'

import os
import sys
from urllib import splitport
from urlparse import urlparse
import xml.etree.ElementTree as etree
from lower_host_lib import is_only_url


HTTP_LIST, HTTPS_LIST, HTTP_PORTS, IP_BLOCK = [], [], [], []
IP_HTTPS_LIST, IP_FTP_LIST, OTHER_URLS, FULLWHITELIST = [], [], [], []
SCHEMES_REST = ('http', 'https', 'ftp',)


def filter_by_scheme(url_list, scheme):
    """ Даёт список URL по нужному протоколу, проще чем анализ по одному """
    return [url for url in url_list if urlparse(url).scheme == scheme]


def pick_port(url):
    """ Извлечение порта из URL, если нет - вернёт None """
    return splitport(urlparse(url).netloc)[1]


def process_http(urls_http):
    """ Логика с самым основным - http url'ами """
    whitelisted = False
    for url in list(urls_http):
        if url.rstrip('/') in FULLWHITELIST or url in FULLWHITELIST:
            urls_http.remove(url)
            whitelisted = True
            continue
        if is_only_url(url):
            urls_http = [url.rstrip('/')]
            break
    return urls_http, whitelisted


def process_rest(urls_rest, urls_http, ip_list, port_list_rest):
    """ Обработка других URL, не столь критично, как http, но тоже важно """
    for url in urls_rest:
        url_parsed = urlparse(url)
        domain, port = splitport(url_parsed.netloc)

        if url_parsed.scheme == "https":
            HTTPS_LIST.append(domain)
            urls_http.append(url.replace('https', 'http', 1))
            continue

        if url_parsed.scheme == "ftp":
            IP_FTP_LIST.extend(ip_list)
            continue

        for port in port_list_rest:
            for ipv4 in ip_list:
                OTHER_URLS.append(ipv4 + " " + port)


def process_elem(elem):
    """ Обрабатывает одну запись в реестре. Распихивает её данные по глобальным
    спискам, предварительно сформировав свои """

    domain_list = [unicode(d.text).strip() for d in elem.findall('domain')]

    ip_list = [ip.text for ip in elem.findall('ip') + elem.findall('ipSubnet')]

    urls = [unicode(url.text).strip() for url in elem.findall('url')]
    urls_http = filter_by_scheme(urls, 'http')
    urls_https = filter_by_scheme(urls, 'https')
    urls_ftp = filter_by_scheme(urls, 'ftp')
    urls_rest = [u for u in urls if urlparse(u).scheme not in SCHEMES_REST]

    port_list_http = [port for port in map(pick_port, urls_http) if port]
    port_list_rest = [port for port in map(pick_port, urls_rest) if port]
    urls_http, whitelisted = process_http(urls_http)
    process_rest(urls_rest, urls_http, ip_list, port_list_rest)
    HTTP_PORTS.extend(port_list_http)

    # пытаемся избежать полной блокировки ip создав urls_http из других источников
    if urls_https:
        HTTPS_LIST.extend(domain_list)
        urls_http.extend([url.replace('https', 'http', 1) for url in urls_https])
        urls_http, whitelisted = process_http(urls_http)
        IP_HTTPS_LIST.extend(ip_list)
    if not urls_https and os.environ.get('BLOCK_HTTPS_IP_EVEN_IF_HTTP') == '1':
        IP_HTTPS_LIST.extend(ip_list)

    if not urls_http and domain_list and not whitelisted:
        urls_http = ["http://" + d for d in domain_list]
        IP_HTTPS_LIST.extend(ip_list)
        HTTPS_LIST.extend(domain_list)

    if urls_ftp:
        IP_FTP_LIST.extend(ip_list)

    if not urls_http:
        IP_BLOCK.extend(ip_list)

    HTTP_LIST.extend(urls_http)


def file_to_list(filename):
    """ Возвращает список строк из файла """
    new_list = []
    if os.path.isfile(filename):
        with open(filename, 'r') as url_file:
            return [unicode(s, 'utf-8').strip() for s in url_file.xreadlines()]
    return new_list


def print_list(list_structure):
    """ Выводит список на экран с нужными префиксами и комментом-описанием """
    item_list, describe, prefix = list_structure
    print "#", describe
    for item in sorted(set(item_list)):
        print prefix, item.encode('utf-8')


def print_lists():
    """ Печатает все списки на экран """
    lists = (
        (HTTP_LIST, "urls to load", "add_url"),
        (HTTPS_LIST, "https domains to resolv", "need_resolv"),
        (IP_BLOCK, "ip to full block", "add_ip_block"),
        (IP_HTTPS_LIST, "ip to block on https", "add_ip_https"),
        (HTTP_PORTS, "http ports", "add_http_port"),
        (IP_FTP_LIST, "ftp ip", "add_ip_ftp"),
        (OTHER_URLS, "other urls by ip + port", "add_ip_port"),
    )
    for list_structure in lists:
        print_list(list_structure)


def process_dump_xml(dump):
    """ Обрабатывает содержимое dump.xml """
    for entry in etree.parse(dump).getroot():
        process_elem(entry)

if __name__ == '__main__':
    FULLWHITELIST = file_to_list('/usr/local/Reductor/lists/our.fullwhitelist')
    DUMP = "/usr/local/Reductor/reductor_container/gost-ssl/php/dump.xml"
    # DUMP = "reductor_container/gost-ssl/php/dump.xml"
    if len(sys.argv) == 2:
        DUMP = sys.argv[1]
    process_dump_xml(DUMP)
    print_lists()
