#!/bin/bash

. /etc/init.d/functions
. /usr/local/Reductor/etc/const
. $CONFIG

TMPLT=$CONTRIB/tmplts/nginx.conf
TMPLT_BLOCK_PAGE=$CONTRIB/tmplts/nginx_block.conf
NGINX_INIT=/etc/rc.d/init.d/nginx
NGINX_CONF=/etc/nginx/conf.d/reductor.conf
NGINX_CONF_BLOCK_PAGE=/etc/nginx/conf.d/reductor_blockpage.conf

admin_ip_bash2nginx() {
	admin['ip']="${admin['ip']//[,;]/ }"
	for ip in ${admin['ip']:-}; do
		echo -e "\tallow $ip;"
	done
}

checks() {
	[ ! -f $TMPLT ] && echo "No nginx.conf tmplt $TMPLT" && return 1
	[ ! -x $NGINX_INIT ] && return 2
	[ -z "${admin['ip']// /}" ] && return 3
	return 0
}

generate() {
	local WEB_INTERFACE_IP="${admin['web_ip']:-0.0.0.0}"
	local WEB_INTERFACE_PORT="${admin['web_port']:-80}"
	m4 -D__ALLOW__="$(admin_ip_bash2nginx)" -D__web_ui_ip_port__="$WEB_INTERFACE_IP:$WEB_INTERFACE_PORT" $TMPLT > $NGINX_CONF || exit 4
	if [ "${filter['forbidden_url_local']:-0}" == '1' ] && [ "${filter['dns']:-0}" = '1' ] && [ -n "${filter['dns_ip']}" ]; then
		if ! ip -4 a | grep -q "${filter['dns_ip']}"; then
			$SETCOLOR_FAILURE
			echo "  - Заглушка на собственном сервере + фильтрация по DNS подразумевают наличие IP адреса "${filter['dns_ip']}" на этом сервере."
			echo "    Он не был обнаружен, так что пропускаем генерацию конфигов nginx для страницы-заглушки"
			$SETCOLOR_NORMAL
			rm -f "$NGINX_CONF_BLOCK_PAGE"
			return 0
		fi
		if [ -f /etc/nginx/ssl/cert.pem -a -f /etc/nginx/ssl/cert.key ]; then
			m4 -DSSL=1 -D__listen_ip__="${filter['dns_ip']}" $TMPLT_BLOCK_PAGE > $NGINX_CONF_BLOCK_PAGE
		else
			$SETCOLOR_FAILURE
			echo "  - WARNING: так как отсутствует /etc/nginx/ssl/cert.pem или /etc/nginx/ssl/cert.key заглушка запускается без поддержки SSL"
			$SETCOLOR_NORMAL
			m4 -D__listen_ip__="${filter['dns_ip']}" $TMPLT_BLOCK_PAGE > $NGINX_CONF_BLOCK_PAGE
		fi

		if [ "$WEB_INTERFACE_PORT" == '80' -a "$WEB_INTERFACE_IP" == '0.0.0.0' ]; then
			$SETCOLOR_FAILURE
			echo "  - WARNING: так как включена заглушка для DNS и не настроен адрес/порт веб-интерфейса, он будет недоступен"
			$SETCOLOR_NORMAL
			rm -f $NGINX_CONF
		fi
	fi
}

[ -f /etc/nginx/conf.d/default.conf ] && rm -f /etc/nginx/conf.d/default.conf
checks && generate
