#/usr/bin/env python
# coding:utf-8
import os
from unittest import TestCase
from lower_host_lib import *
from lower_host import *



class TestHelpFunctions(TestCase):

    @staticmethod
    def test_pick_get():
        input_data = [
            u'http://testing.ru',
            u'http://багофичи.рф/гет-запрос',
            u'http://request.good/request_good',
            u'http://request.good?request_bad',
        ]
        expected = [
            u'',
            u'/гет-запрос',
            u'/request_good',
            u'?request_bad',
        ]
        for num, url in enumerate(input_data):
            get_expected = expected[num]
            get_real = pick_get(url)
            assert get_expected == get_real

    @staticmethod
    def test_pick_host():
        input_data = [
            u'http://testing.ru',
            u'http://багофичи.рф/гет-запрос',
            u'http://testing.ru?request'
        ]
        expected = [
            u'testing.ru',
            u'багофичи.рф',
            u'testing.ru'
        ]
        for num, url in enumerate(input_data):
            try:
                host_expected = expected[num]
                host_real = pick_host(url)
                assert host_expected == host_real
            except AssertionError as e:
                print expected[num], pick_host(url), url
                raise e

    @staticmethod
    def test_domain_only_urls():
        input_data = set([
            u'http://domain.ru/kek/',
            u'http://domain.ru/kek/lol',
            u'http://slash.no',
            u'http://slash.yes',
            u'http://slash.yes/',
            u'http://slash.only/',
        ])
        expected = set([
            u'http://slash.no',
            u'http://slash.yes',
            u'http://slash.only'
        ])
        assert expected == domain_only_urls(input_data)


class TestTransforms(TestCase):

    @staticmethod
    def test_get_question_remove():
        input_data = u'http://example.com/?'
        expected = u'http://example.com'
        output = all_transforms_line(input_data)
        assert expected in output

    @staticmethod
    def test_url_with_ports():
        input_data = u'http://padonki.org:8080/creo.do?creoId=14262'
        output = all_transforms_line(input_data)
        assert input_data in output

    @staticmethod
    def test_last_dot_in_get():
        input_data = u'http://example.com/gaminators/.'
        expected = u'http://example.com/gaminators'
        output = all_transforms_line(input_data)
        assert expected in output

    @staticmethod
    def test_domain_change():
        input_data = set([
            u'http://vk.com/cybergetrequest',
            u'http://vkontakte.ru/cyberget',
            u'http://normal.url/cyberget',
            u'http://normal.url/vk.com/lol'
        ])
        expected = set([
            u'http://vk.com/cybergetrequest',
            u'http://vkontakte.ru/cybergetrequest',
            u'http://vkontakte.ru/cyberget',
            u'http://vk.com/cyberget',
            u'http://normal.url/cyberget',
            u'http://normal.url/vk.com/lol'
        ])
        assert domain_change(input_data) == expected

    @staticmethod
    def test_reduce_length():
        input_data = set([
            u'http://short.url/',
            (u'http://long.url/' + ('xyz' * 10 + '/') * 23)
        ])
        expected = set([
            u'http://short.url/',
            (u'http://long.url/' + ('xyz' * 10 + '/') * 18).rstrip('/')
        ])
        assert reduce_length(input_data) == expected

    @staticmethod
    def test_latest_bugs():
        input_data = [
            u'http://lurkmore.to/index.php?title=Конопля/Способы_потребления&oldid= 2388276',
        ]
        expected = [
            u'http://lurkmore.to/index.php?title=Конопля',
        ]
        for index in xrange(len(expected)):
            output = all_transforms_line(input_data[index])
            assert expected[index] in output

    @staticmethod
    def test_sharp_remove():
        input_data = set([
            u'http://normal.url',
            u'http://with.sharp/in#request'
        ])
        expected = set([
            u'http://normal.url',
            u'http://with.sharp/in',
            u'http://with.sharp/in#request'
        ])
        assert sharp_remove(input_data) == expected

    @staticmethod
    def test_valid_http_only():
        input_data = set([
            u'http://normal.url',
            u'https://normal.url',
            u'https://bad.url',
            u'# comment',
            u'http://'
        ])
        expected = set([
            u'http://normal.url'
        ])
        assert valid_http_only(input_data) == expected

    @staticmethod
    def test_fulldomain_reduce():
        """ TODO в будущем нужно восстановить """
        input_data = set([
            u'http://domain.ru/',
            u'http://domain.ru/kek/',
            u'http://domain.ru/kek/lol',
            u'http://slash.no',
            u'http://slash.yes',
            u'http://slash.yes/',
            u'http://just.url/kek/lol'
        ])
        expected = set([
            u'http://domain.ru',
            u'http://slash.no',
            u'http://slash.yes',
            u'http://just.url/kek/lol'
        ])
        assert expected == fulldomain_reduce(input_data)

    @staticmethod
    def test_remove_bad_symbol_parts():
        input_data = set([
            u'http://domain.ru/лол/Р',
            u'http://domain.ru/kek/Р',
            u'http://domain.ru/kek/nya',
            u'http://domain.ru/very/long/url/with/that/Р/symbol',
            u'http://domain.ru/with /space',
            u'http://domain.ru/with/space /lol',
        ])
        expected = set([
            u'http://domain.ru/лол',
            u'http://domain.ru/kek',
            u'http://domain.ru/with',
            u'http://domain.ru/with/space/lol',
            u'http://domain.ru/лол/',
            u'http://domain.ru/kek/',
            u'http://domain.ru/kek/nya',
            u'http://domain.ru/very/long/url',
            u'http://domain.ru/very/long/url/with/that//symbol',
        ])
        assert expected == remove_bad_symbols_parts(input_data)

    @staticmethod
    def test_idna():
        input_data = set([
            u'http://normal.url',
            u'http://кириллица.рф',
            u'http://кириллица.рф/just-request',
            u'http://combined.рф',
            u'http://with.request/кириллический-запрос'
        ])

        expected = set([
            u'http://xn--80apaahia1b8c.xn--p1ai',
            u'http://with.request/кириллический-запрос',
            u'http://normal.url',
            u'http://combined.xn--p1ai',
            u'http://xn--80apaahia1b8c.xn--p1ai/just-request'
        ])
        assert expected == idna(input_data)

    @staticmethod
    def test_fqdn():
        """ отрезать точки в конце, если есть """
        input_data = set([
            u'http://norm.url',
            u'http://norm.url.',
            u'http://fqdn.url.',
            u'http://norm.url/get_request',
            u'http://fqdn.url./get_request',
            u'http://norm.url?bad_request',
            u'http://fqdn.url?bad_request',
        ])

        expected = set([
            u'http://norm.url',
            u'http://fqdn.url',
            u'http://norm.url/get_request',
            u'http://fqdn.url/get_request',
            u'http://norm.url?bad_request',
            u'http://fqdn.url?bad_request',
        ])
        assert expected == fqdn(input_data)

    @staticmethod
    def test_lower():
        """ копировать lower_host, но не добавлять lower_get """
        input_data = set([
            u'http://norm.url',
            u'http://UPPER.URL',
            u'http://Mixed.Url',
            u'http://norm.url/get',
            u'http://UPPER.URL/UPPER.GET',
            u'http://UPPER.URL/lower.get',
            u'http://КИРИЛЛИЧЕСКИЙ.РФ',
            u'http://КИРИЛЛИЧЕСКИЙ.РФ?zaproS'
        ])
        expected = set([
            u'http://norm.url',
            u'http://UPPER.URL',
            u'http://upper.url',
            u'http://Mixed.Url',
            u'http://mixed.url',
            u'http://norm.url/get',
            u'http://UPPER.URL/UPPER.GET',
            u'http://upper.url/UPPER.GET',
            u'http://UPPER.URL/lower.get',
            u'http://upper.url/lower.get',
            u'http://КИРИЛЛИЧЕСКИЙ.РФ',
            u'http://кириллический.рф',
            u'http://КИРИЛЛИЧЕСКИЙ.РФ?zaproS',
            u'http://кириллический.рф?zaproS'
        ])
        assert expected == lower(input_data)

    @staticmethod
    def test_slash():
        """
        / отрезать можно только в конце домена
        добавлять нужно перед ? если до этого не было /
        добавлять в конец длинных GET-запросов вероятно тоже надо
        """
        input_data = set([
            u'http://normal.url',
            u'http://with.slash/',
            u'http://normal.url/get',
            u'http://url.with?bad_request',
            u'http://url.without/?bad_request',
            u'http://img.booru.org/allgirl//images/41/13302d8ede5038c627483c2aacae9cb3e22e349a.jpg'
        ])
        expected = set([
            u'http://normal.url',
            u'http://with.slash',
            u'http://normal.url/get',
            u'http://url.with?bad_request',
            u'http://url.with/?bad_request',
            u'http://url.without/?bad_request',
            u'http://url.without?bad_request',
            u'http://img.booru.org/allgirl/images/41/13302d8ede5038c627483c2aacae9cb3e22e349a.jpg',
            u'http://img.booru.org/allgirl/images/41/13302d8ede5038c627483c2aacae9cb3e22e349a.jpg/'
        ])
        assert expected == slash(input_data)

    @staticmethod
    def test_www():
        """ www нужно отрезать только для доменов 5+ уровня """
        input_data = set([
            u'http://normal.url',
            u'http://both.url',
            u'http://www.both.url',
            u'http://www.only.url',
            u'http://www.much.items.url.blin/request'
        ])
        expected = set([
            u'http://normal.url',
            u'http://both.url',
            u'http://only.url',
            u'http://www.much.items.url.blin/request',
            u'http://much.items.url.blin/request'
        ])
        assert www(input_data) == expected

    @staticmethod
    def test_cp1251():
        input_data = set([
            u'http://good.item/lol',
            u'http://notgood.item/лол'
        ])
        output = cp1251(input_data)

    @staticmethod
    def test_no_remove_encoded_spaces():
        input_data = u'http://example.com/m/op%20hay/lalalay'
        expect = u'http://example.com/space'
        assert expect not in all_transforms_line(input_data)

    @staticmethod
    def test_url_quote_unquote():
        """ экранирование должно затрагивать только кириллицу """
        input_data = set([
            u'http://good.item',
            u'http://good.item/lol',
            u'http://cyrillic.get/суперурл'
        ])

        expected = set([
            u'http://good.item',
            u'http://good.item/lol',
            u'http://cyrillic.get/суперурл',
            u'http://cyrillic.get/%D1%81%D1%83%D0%BF%D0%B5%D1%80%D1%83%D1%80%D0%BB'
        ])
        assert expected == url_quote_unquote(input_data)

    @staticmethod
    def test_url_cp1251_quoted():
        input_data = set([
            u'http://suicide-forum.com/showthread.php?157-%C4%EB%FF-%F2%E5%F5-%EA%F2%EE-%F5%EE%F7%E5%F2-%F3%EC%E5%F0%E5%F2%FC',
            u'http://suicide-forum.com/showthread.php?2507-%CC%E0%F1%F1%EE%E2%FB%E9-%F1%F3%E8%F6%E8%E4!',
        ])
        expected = [
            u'http://suicide-forum.com/showthread.php?157-Для-тех-кто-хочет-умереть',
            u'http://suicide-forum.com/showthread.php?2507-%D0%9C%D0%B0%D1%81%D1%81%D0%BE%D0%B2%D1%8B%D0%B9-%D1%81%D1%83%D0%B8%D1%86%D0%B8%D0%B4!'
        ]
        output = url_quote_unquote(input_data)
        for expect in expected:
            assert expect in output

    @staticmethod
    def test_muzofon():
        """ из-за %20 не должен блокироваться весь /search/ """
        input_data = u'http://xx.ru/search/Коловрат%20Не%20покупай%20у%20чурок'
        unexpected = [
            u'http://xx.ru/search',
            u'http://xx.ru/search/',
        ]
        output = all_transforms_line(input_data)
        for unexpect in unexpected:
            assert unexpect not in output


    @staticmethod
    def test_wget_quote():
        """ экранирование wget слегка отличается от остальных """
        input_data = set([
            u'http://suicide-forum.com/showthread.php?2507-Массовый-суицид!'
        ])
        expected = u'http://suicide-forum.com/showthread.php?2507-%D0%9C%D0%B0%D1%81%D1%81%D0%BE%D0%B2%D1%8B%D0%B9-%D1%81%D1%83%D0%B8%D1%86%D0%B8%D0%B4!'
        assert expected in url_quote_unquote(input_data)


class TestTotal(TestCase):

    @staticmethod
    def test_print_decode_file():
        test_file = '/usr/local/Reductor/lists/rkn.lists'
        if not os.path.isfile(test_file):
            if not os.path.isdir('/proc'):
                test_file = 'lists/main.minjust'
            if not os.path.isfile(test_file):
                test_file = '../lists/main.minjust'
        if not os.path.isfile(test_file):
            assert False
        print_decode_file(open(test_file))

    @staticmethod
    def test_all_transforms():
        input_data = set([
            u'http://normal.url/get',
            u'http://1226608.sites.myregisteredsite.com/seantheartist/archived/01032012/0204-full/F04D-ScienceOfHanging-1.html'
        ])
        for url in input_data:
            assert url in all_transforms_line(url)
