#!/bin/bash

. /usr/local/Reductor/etc/const
. $CONFIG
. $BINDIR/list_processing.sh

set -eu

CUSTOM_EXIT=1
LOCKDIR=/var/lock/reductor/
LOCKFILE=$LOCKDIR/prepare_lists.sh.lock

trap __exit EXIT

__custom_exit() {
	flock -u 3
}

debug=0

prepare() {
	mkdir -p $TMPDIR $LOCKDIR
	exec 3>$LOCKFILE
	log "- Захватываем lock" >> $LOGFILE
	flock -x 3
	log "- Обрабатываем" >> $LOGFILE
	zero_files "${IP_FILES[@]}" "${URL_FILES[@]}"
}

normalize() {
	local categorized="$1.all"
	local filename="${1##*/}"
	if [ ! -f "$categorized" ]; then
		categorized="$1"
	fi
	debug "normalize categorized=$categorized"
	if grep "[%А-Яа-я]" "$categorized" > "$TMPDIR/$filename.wget"; then
		while read -r line; do
			LANG=ru_RU.UTF-8 "$BINDIR/wget_quote" "$line" 2>/dev/null
		done < "$TMPDIR/$filename.wget" > "$TMPDIR/$filename.wget_new"
		LANG='' sort -u "$TMPDIR/$filename.wget_new" "$categorized" > "$TMPDIR/$filename.wget"
		rm -f "$TMPDIR/$filename.wget_new"
	else
		cp "$categorized" "$TMPDIR/$filename.wget"
	fi
	# possible bug
	LANG=ru_RU.UTF-8 $BINDIR/lower_host.py < "$TMPDIR/$filename.wget" > "$TMPDIR/$filename.normal"
	rm -f "$TMPDIR/$filename.wget"
	mv "$TMPDIR/$filename.normal" "$1"
	debug "post normalize $@"
}

whitelisting_fulldomain() {
	local whitelist="$1"
	local list="$2"
	local domain=''
	local noload=$whitelist.entries
	> $noload
	while read -r url; do
		domain="$(echo "$url" | cut -d '/' -f3)"
		grep -F "http://$domain" "$list" >> "$noload"
		grep -F -v "http://$domain" "$list" > "$list.tmp"
		mv -f "$list.tmp" "$list"
	done < $whitelist
}

use_hook() {
	hook="$HOOKDIR/${0##*/}"
	if [ -x "$hook" ]; then
		. "$hook"
	fi
}

client_prepare_hook() {
	return 0
}

# убираем www и uppercase host
normalize_all() {
	debug normalize_all
	normalize $LISTDIR/http.load &
	normalize $LISTDIR/http.noload &
	normalize $LISTDIR/http_full.noload &
	mkdir -p $HOOKDIR/custom_url_lists/
	for list in $(find $HOOKDIR/custom_url_lists/ -type f); do
		normalize $list &
	done
	wait
}

main() {
	client_prepare_hook
	echo "- Обработка списков"
	prepare
	echo "- Агрегация списков"
	categorize_url
	categorize_ip
	echo "- Обработка URL"
	normalize_all
	echo "- Применение белых списков"
	whitelisting_url
	$BINDIR/update_signatures.sh &
	resolv fast &
	wait
	categorize_ip
	whitelisting_ip
}

use_hook
"${@:-main}"
